@@ -1,23 +1,49 @@
 #!/usr/bin/env python
-from typing import Optional
 import itertools
-import pandas as pd
-import numpy as np
-from typing import Iterable
-from pathlib import Path
 import json
-from pathlib import Path
+import re
 import sys
+from pathlib import Path
+from typing import Dict, Iterable, Optional
+
+import numpy as np
+import pandas as pd
 
 __all__ = ["has_visdata", "has_logs", "main", "VISDATA_FOLDER"]
 
+BASE_IGNORE_SET = {"agent_types", "agent_params", "non_competitors", "non_competitor_params"}
+BASE_WORLD_PARAMS_IGNORE_SET = {
+    "__dir_name",
+    "agent_params",
+    "agent_types",
+    "name",
+    "catalog_prices",
+    "profiles",
+    "exogenous_contracts",
+    "active_lines",
+    "input_processes",
+    "output_processes",
+    "expected_income",
+    "expected_income_per_process",
+    "expected_income_per_step",
+    "expected_n_products",
+    "input_quantities",
+    "output_quantities",
+    "product_prices",
+    "process_inputs",
+    "process_outputs",
+    "profit_basis",
+}
+ASSIGNED_IGNORE_SET = BASE_IGNORE_SET
+ASSIGNED_WORLD_PARAMS_IGNORE_SET = BASE_WORLD_PARAMS_IGNORE_SET
 VISDATA_FOLDER = "_visdata"
 # tournament files
 SCORES_FILE = "scores.csv"
-CONFIGS_FILE = "assigned_configs.json"  # has n_steps, __dir_name
+ASSIGNED_CONFIGS_FILE = "assigned_configs.json"  # has n_steps, __dir_name
+BASE_CONFIGS_FILE = "base_configs.json"  # has n_steps, __dir_name
 # WORLD_STATS_FILE = "stats.csv" # has path and world
 
-TOURNAMENT_REQUIRED = [SCORES_FILE, CONFIGS_FILE]
+TOURNAMENT_REQUIRED = [SCORES_FILE, ASSIGNED_CONFIGS_FILE]
 
 # single world files
 CONTRACTS_FILE = ("contracts.csv", "contracts_full_info.csv")
@@ -37,16 +63,17 @@ PATHMAP = dict()
 def has_logs(folder: Path, check_tournament=True, check_world=True) -> bool:
     if check_world:
         for f in WORLD_REQUIRED:
-            if not (folder/ f).exists():
+            if not (folder / f).exists():
                 return False
         return True
     if check_tournament:
         for f in TOURNAMENT_REQUIRED:
-            if not (folder/ f).exists():
+            if not (folder / f).exists():
                 return False
         return True
     return True
 
+
 def adjust_type_names(df):
     if df is None or len(df) == 0:
         return df
@@ -56,16 +83,21 @@ def adjust_type_names(df):
         df[c] = df[c].str.split(".").str[0] + "." + df[c].str.split(".").str[-1]
     return df
 
+def adjust_column_names(df):
+    if df is None or len(df) == 0:
+        return df
+    return df.rename(dict(config_id="config", tournament_id="tournament", world_id="world", group_id="group"), axis=1)
+
 
 def nonzero(f):
     return f.exists() and f.stat().st_size > 0
 
 
-def get_folders(base_folder, main_file, required):
+def get_folders(base_folder, main_file, required, ignore=None):
     return [
         _.parent
         for _ in base_folder.glob(f"**/{main_file}")
-        if all(nonzero(f) for f in [_.parent / n for n in required])
+        if (not ignore or not re.match(ignore, _.name)) and all(nonzero(f) for f in [_.parent / n for n in required])
     ]
 
 
@@ -77,67 +109,97 @@ def is_world(base_folder):
     return all(nonzero(f) for f in [base_folder / n for n in WORLD_REQUIRED])
 
 
-def get_torunaments(base_folder):
-    return get_folders(base_folder, main_file=CONFIGS_FILE, required=TOURNAMENT_REQUIRED)
+def get_torunaments(base_folder, ignore=None):
+    return get_folders(base_folder, main_file=ASSIGNED_CONFIGS_FILE, required=TOURNAMENT_REQUIRED, ignore=ignore)
 
 
-def get_worlds(base_folder):
-    return get_folders(base_folder, main_file=AGENTS_FILE, required=WORLD_REQUIRED)
+def get_worlds(base_folder, ignore=None):
+    return get_folders(base_folder, main_file=AGENTS_FILE, required=WORLD_REQUIRED, ignore=ignore)
 
 
-def parse_tournament(path, t_indx, base_indx):
-    configs = json.load(open(path / CONFIGS_FILE))
-    if not configs:
-        return None, None, None
-    scores = pd.read_csv(path / SCORES_FILE).to_dict("records") # typing: none
+def parse_tournament(
+    path,
+    t_indx,
+    base_indx,
+    config_ignore_set=BASE_IGNORE_SET,
+    assigned_config_ignore_set=ASSIGNED_IGNORE_SET,
+    base_world_ignore_set=BASE_WORLD_PARAMS_IGNORE_SET,
+    assigned_world_ignore_set=ASSIGNED_WORLD_PARAMS_IGNORE_SET,
+):
+    base_configs = json.load(open(path / BASE_CONFIGS_FILE))
+    if not base_configs:
+        return None, None, None, None
+    configs = []
+    config_groups = []
+    group_map = dict()
+    def copy_with_ignore(d, rename_dict, ignore_set, recurse_set):
+        c = dict()
+        for k, v in d.items():
+            if k in ignore_set:
+                continue
+            if k in recurse_set:
+                c = {**c, **copy_with_ignore(v, dict(), ignore_set, recurse_set)}
+                continue
+            if k in rename_dict.keys():
+                c[rename_dict[k]] = v
+                continue
+            c[k] = v
+        return c
+
+    recurse_set = ["world_params", "__exact_params", "info"]
+    for aconfig in base_configs:
+        conf_group_id = ""
+        config_set = []
+        for c in aconfig:
+            conf_group_id = (conf_group_id + ":" + c["config_id"]) if conf_group_id else c["config_id"]
+            d = copy_with_ignore(c, dict(config_id="id"), config_ignore_set.union(base_world_ignore_set), recurse_set)
+            config_set.append(d)
+        config_groups.append(dict(id=conf_group_id))
+        for c in config_set:
+            c["group"] = conf_group_id
+            configs.append(c)
+            group_map[c["id"]] = conf_group_id
+
+    assigned_configs = json.load(open(path / ASSIGNED_CONFIGS_FILE))
+    if not assigned_configs:
+        return None, None, None, None
+    scores = pd.read_csv(path / SCORES_FILE).to_dict("records")  # typing: none
     if not scores:
-        return None, None, None
+        return None, None, None, None
     worlds = []
     world_indx = dict()
     world_names = set()
     agents = []
-    for i, _ in enumerate(configs):
-        if i > MAXWORLDS:
+    for i, aconfig in enumerate(assigned_configs):
+        if i > MAXWORLDS or not aconfig:
             break
-        c = _[0]
-        world_names.add(c["world_params"]["name"])
-        p = c["__dir_name"]
-        for k, v in PATHMAP.items():
-            p = p.replace(k, v)
-        worlds.append(
-            dict(
-                id=i + base_indx,
+        for c in aconfig:
+            world_names.add(c["world_params"]["name"])
+            p = c["__dir_name"]
+            for k, v in PATHMAP.items():
+                p = p.replace(k, v)
+            d = dict(
+                id=c["world_params"]["name"],
                 path=p,
-                name=c["world_params"]["name"],
-                n_steps=c["world_params"]["n_steps"],
-                n_processes=c["world_params"]["n_processes"],
                 n_agents=len(c["is_default"]),
                 tournament=path.name,
                 tournament_indx=t_indx,
             )
-        )
-        world_indx[worlds[-1]["id"]] = worlds[-1]["name"]
-        _, wa = get_basic_world_info(Path(p), path.name)
-        agents.append(wa)
+            d = {
+                **copy_with_ignore(c, dict(), assigned_config_ignore_set.union(assigned_world_ignore_set), recurse_set),
+                **d,
+            }
+            d["name"] = c["world_params"]["name"]
+            worlds.append(d)
+            world_indx[worlds[-1]["id"]] = worlds[-1]["name"]
+            _, wa = get_basic_world_info(Path(p), path.name, worlds[-1]["config_id"], group_map.get(worlds[-1]["config_id"], "none"))
+            if not _:
+                continue
+            agents.append(wa)
 
     agents = pd.concat(agents)
 
-    # for i, s in enumerate(scores):
-    #     if s["world"] not in (world_names):
-    #         continue
-    #     agents.append(
-    #         dict(
-    #             id=i + base_indx,
-    #             name=s["agent_id"],
-    #             type=s["agent_type"],
-    #             # id=s["agent_id"],
-    #             final_score=s["score"],
-    #             world=s["world"],
-    #             world_id=world_indx.get(s["world"], None),
-    #         )
-    #     )
-    # agents = pd.DataFrame.from_records(agents)
-    return worlds, agents
+    return config_groups, configs, worlds, agents
 
 
 def parse_world(path, tname, wname, nsteps, agents, w_indx, base_indx):
@@ -147,19 +209,19 @@ def parse_world(path, tname, wname, nsteps, agents, w_indx, base_indx):
     # except:
     #     winfo = None
     if (path / AGENTS_JSON_FILE).exists():
-            scored_agents = set(agents["name"].to_list())
-            ag_dict = json.load(open(path / AGENTS_JSON_FILE))
-            for k, v in ag_dict.items():
-                if k in ("BUYER", "SELLER"):
-                    v["final_score"] = float("nan")
-                elif k in scored_agents:
-                    a, b = stats[f"score_{k}"].values[-1], float(agents.loc[agents["name"]==k, "final_score"].values)
-                    assert a == b
-                    v["final_score"] = float(agents.loc[agents["name"]==k, "final_score"].values)
-                else:
-                    v["final_score"] = stats[f"score_{k}"].values[-1]
-            world_agents = pd.DataFrame.from_records(list(ag_dict.values()))
-            world_agents["official_score"] = world_agents["name"].isin(scored_agents)
+        scored_agents = set(agents["name"].to_list())
+        ag_dict = json.load(open(path / AGENTS_JSON_FILE))
+        for k, v in ag_dict.items():
+            if k in ("BUYER", "SELLER"):
+                v["final_score"] = float("nan")
+            elif k in scored_agents:
+                a, b = stats[f"score_{k}"].values[-1], float(agents.loc[agents["name"] == k, "final_score"].values)
+                assert a == b
+                v["final_score"] = float(agents.loc[agents["name"] == k, "final_score"].values)
+            else:
+                v["final_score"] = stats[f"score_{k}"].values[-1]
+        world_agents = pd.DataFrame.from_records(list(ag_dict.values()))
+        world_agents["official_score"] = world_agents["name"].isin(scored_agents)
     else:
         world_agents = agents
         world_agents["official_score"] = True
@@ -611,7 +673,7 @@ def parse_world(path, tname, wname, nsteps, agents, w_indx, base_indx):
             col_name = f"{n}_{p}"
             colnames.append(col_name)
         x = stats.loc[:, colnames].reset_index().rename(columns=dict(index="step"))
-        x.columns = ["_".join(_.split("_")[:-1]) if _.endswith(f"_{p}") else _ for _ in x.columns ]
+        x.columns = ["_".join(_.split("_")[:-1]) if _.endswith(f"_{p}") else _ for _ in x.columns]
         if len(x):
             x["relative_time"] = x["step"] / nsteps
         else:
@@ -642,18 +704,26 @@ def parse_world(path, tname, wname, nsteps, agents, w_indx, base_indx):
     )
 
 
-def get_basic_world_info(path, tname):
-    stats = pd.read_csv(path / STATS_FILE, index_col=0).to_dict("list")
-    adata = json.load(open(path / AGENTS_JSON_FILE))
-    winfo = json.load(open(path / INFO_FILE))
-    worlds = [dict(name=path.name, tournament=tname, tournament_indx=0
-    , path=path, n_steps=winfo["n_steps"])]
+def get_basic_world_info(path, tname, gname, cname):
+    try:
+        stats = pd.read_csv(path / STATS_FILE, index_col=0).to_dict("list")
+        adata = json.load(open(path / AGENTS_JSON_FILE))
+        winfo = json.load(open(path / INFO_FILE))
+    except:
+        print(f"FAILED {path.name}", flush=True)
+        return [], None
+    worlds = [dict(name=path.name, tournament=tname, config=cname, group=gname, tournament_indx=0, path=path, n_steps=winfo["n_steps"])]
     agents = []
     definfo = winfo.get("is_default", None)
+    agent_key = None
+    for k in ("agent_initial_balances", "agent_profiles", "agent_inputs", "agent_outputs", "agent_processes"):
+        if k in winfo.keys():
+            agent_key = k
+            break
     if definfo:
-        is_default = dict(zip(winfo["agent_initial_balances"].keys(), definfo))
+        is_default = dict(zip(winfo[agent_key].keys(), definfo))
     else:
-        is_default = dict(zip(winfo["agent_initial_balances"].keys(), itertools.repeat(False)))
+        is_default = dict(zip(winfo[agent_key].keys(), itertools.repeat(False)))
     for i, (aname, info) in enumerate(adata.items()):
         if f"score_{aname}" not in stats.keys():
             continue
@@ -664,13 +734,13 @@ def get_basic_world_info(path, tname):
         if "costs" in aginfo.keys():
             aginfo["cost"] = float(np.asarray(aginfo["costs"]).min())
             del aginfo["costs"]
-        dd = dict(id=i, name=aname, world=worlds[0]["name"], tournament=tname, final_score=score, type=info["type"])
+        dd = dict(id=i, name=aname, world=worlds[0]["name"], tournament=tname, group=gname, config=cname, final_score=score, type=info["type"])
         dd = {**dd, **aginfo}
-        agents.append( dd)
+        agents.append(dd)
     return worlds, pd.DataFrame.from_records(agents)
 
 
-def get_data(base_folder):
+def get_data(base_folder, ignore: Optional[str] = None):
     base_folder = Path(base_folder)
     tournaments, worlds, agents, agent_stats, product_stats, world_stats = (
         [],
@@ -680,39 +750,70 @@ def get_data(base_folder):
         [],
         [],
     )
+    groups, configs = [], []
     contracts, contract_stats, breaches = [], [], []
     negotiations, offers, neg_stats = [], [], []
     if is_tournament(base_folder):
-        paths = get_torunaments(base_folder)
+        paths = get_torunaments(base_folder, ignore)
     elif is_world(base_folder):
         paths = [None]
     else:
-        raise ValueError(f"Folder {str(base_folder)} contains neither tournament nor world logs")
+        paths = get_torunaments(base_folder, ignore)
+        paths += get_worlds(base_folder, ignore)
+        # raise ValueError(f"Folder {str(base_folder)} contains neither tournament nor world logs")
+    none_tournament = dict(id="none", path=base_folder.parent, name="none")
+    none_group = dict(id="none")
+    none_config = dict(id="none", name="none")
+    none_added = False
     for i, t in enumerate(paths):
         indx = i + 1
         base_indx = (i + 1) * 1_000_000
         if t is not None:
-            print(f"Processing {t.name} [{i} of {len(paths)}]", flush=True)
-            tournaments.append(dict(id=indx, path=t, name=t.name))
-            w, a = parse_tournament(t, indx, base_indx)
-            tname = t.name
+            if is_tournament(t):
+                print(f"Tournament {t.name} [{i} of {len(paths)}]", flush=True)
+                tournaments.append(dict(id=indx, path=t, name=t.name))
+                g, con, w, a = parse_tournament(t, indx, base_indx)
+                tname = t.name
+            else:
+                print(f"World {t.name} [{i} of {len(paths)}]", flush=True)
+                if not none_added:
+                    tournaments.append(none_tournament)
+                    none_added = True
+                w, a = get_basic_world_info(base_folder, "none", "none", "none")
+                g = [{k: v for k, v in none_group.items()}]
+                con = [{k: v for k, v in none_config.items()}]
+                tname = "none"
         else:
             print(f"No tournament found ... reading world info")
             tname = "none"
             tournaments.append(dict(id=tname, path=base_folder.parent, name=tname))
-            w, a = get_basic_world_info(base_folder, tname)
+            g = [{k: v for k, v in none_group.items()}]
+            con = [{k: v for k, v in none_config.items()}]
+            w, a = get_basic_world_info(base_folder, tname, "none", "none")
+        for gg in g:
+            gg["tournament"] = tname
+        for cc in con:
+            cc["tournament"] = tname
+        groups += g
+        configs += con
+        if not w:
+            continue
         for j, world in enumerate(w):
-            print(f"\tWorld {world['name']} [{j} of {len(w)}]", flush=True)
             wagents = a.loc[a.world == world["name"]]
-            ag, pr, wo, co, cs, ng, of, ns, br = parse_world(
-                Path(world["path"]),
-                tname,
-                world["name"],
-                world["n_steps"],
-                wagents,
-                base_indx + j + 1,
-                base_indx + j + 1,
-            )
+            try:
+                ag, pr, wo, co, cs, ng, of, ns, br = parse_world(
+                    Path(world["path"]),
+                    tname,
+                    world["name"],
+                    world["n_steps"],
+                    wagents,
+                    base_indx + j + 1,
+                    base_indx + j + 1,
+                )
+            except:
+                print(f"\tParse Error {world['name']}", flush=True)
+                continue
+            print(f"\tWorld {world['name']} [{j} of {len(w)}]", flush=True)
             if ag is not None and len(ag):
                 agent_stats.append(ag)
             if pr is not None and len(pr):
@@ -737,6 +838,8 @@ def get_data(base_folder):
             agents.append(a)
 
     tournaments = pd.DataFrame.from_records(tournaments)
+    groups = pd.DataFrame.from_records(groups)
+    configs = pd.DataFrame.from_records(configs)
     if worlds is not None and len(worlds):
         worlds = pd.concat(worlds, ignore_index=True)
     if agents is not None and len(agents):
@@ -761,6 +864,8 @@ def get_data(base_folder):
         breaches = pd.concat(breaches, ignore_index=True)
     return (
         tournaments,
+        groups,
+        configs,
         worlds,
         agents,
         agent_stats,
@@ -775,16 +880,42 @@ def get_data(base_folder):
     )
 
 
-def main(folder: Path, max_worlds: Optional[int]):
+def map_paths(folder: Path, m: Dict[str, str]):
+    """Maps paths inside all files within the given folder"""
+
+    for fname in folder.glob("**/*"):
+        if fname.is_dir():
+            continue
+        name = fname.name
+        if not name.endswith("json") and not name.endswith("csv"):
+            continue
+        with open(fname, "r") as infile:
+            s = infile.read()
+        for k, v in m.items():
+            s = s.replace(k, v)
+        with open(fname, "w") as outfile:
+            outfile.write(s)
+
+
+def main(folder: Path, max_worlds: Optional[int], ignore: Optional[str] = None, pathmap: Optional[str] = None):
     folder = Path(folder)
     if max_worlds is None:
         max_worlds = float("inf")
     dst_folder = folder / VISDATA_FOLDER
     if dst_folder.exists():
-        raise ValueError(f"Destiantion folder {dst_folder} exists. Delete it if you want to recompile visualization data")
+        raise ValueError(
+            f"Destiantion folder {dst_folder} exists. Delete it if you want to recompile visualization data"
+        )
     folder = Path(folder)
+
+    if pathmap:
+        m = pathmap.split(":")
+        map_paths(folder, dict(zip(m[0::2], m[1::2])))
+
     (
         tournaments,
+        groups,
+        configs,
         worlds,
         agents,
         agent_stats,
@@ -796,12 +927,14 @@ def main(folder: Path, max_worlds: Optional[int]):
         offers,
         neg_stats,
         breaches,
-    ) = get_data(folder)
+    ) = get_data(folder, ignore=ignore)
     dst_folder = folder / VISDATA_FOLDER
     dst_folder.mkdir(parents=True, exist_ok=True)
     for df, name in zip(
         (
             tournaments,
+            groups,
+            configs,
             worlds,
             agents,
             agent_stats,
@@ -816,6 +949,8 @@ def main(folder: Path, max_worlds: Optional[int]):
         ),
         (
             "tournaments",
+            "groups",
+            "configs",
             "worlds",
             "agents",
             "agent_stats",
@@ -830,8 +965,10 @@ def main(folder: Path, max_worlds: Optional[int]):
         ),
     ):
         if df is None or len(df) == 0:
+            print(f"\tDid not find {name}")
             continue
         df = adjust_type_names(df)
+        df = adjust_column_names(df)
         df.to_csv(dst_folder / f"{name}.csv", index=False)
 
 
