@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 from pathlib import Path
 from collections import defaultdict
+from pprint import pprint
 from pprint import pformat
 import random
 import numpy as np
@@ -26,16 +27,20 @@ __all__ = [
     "add_stats_display",
     "load_data",
     "plot_netowrk",
-    "score_distribution"
+    "score_distribution",
+    "score_factors",
 ]
 
 
-@st.cache
+@st.cache(allow_output_mutation=True)
 def load_data(folder: Path, name: str):
     file = folder / f"{name}.csv"
     if not file.exists():
         return None
     data = pd.read_csv(file, index_col=None)
+    if name == "agents":
+        data["is-default"] = data["is_default"].astype(int)
+        data["is-builtin"] = data["type"].str.startswith("scml")
     return data
 
 
@@ -52,6 +57,7 @@ def add_selector(
     check=False,
     check2=False,
     default_check=False,
+    default_choice=None,
 ):
     options = []
     indx = 0
@@ -74,15 +80,22 @@ def add_selector(
     with col1:
         selection_type = st.radio(title, options, index=indx, key=f"{key}_sel_type")
     if selection_type == "none":
-        return ([], combine, overlay) if check or overlay else []
+        return [], combine, overlay
     if selection_type == "all":
-        return (content, combine, overlay) if check or overlay else content
+        return content, combine, overlay
     with col2:
         if selection_type == "some":
-            selector = st.multiselect("", content, key=f"{key}_multi")
-            return (selector, combine, overlay) if check or overlay else selector
-        selector = st.selectbox("", content, key=f"{key}_sel")
-        return ([selector], combine, overlay) if check or overlay else [selector]
+            selector = st.multiselect("", content, key=f"{key}_multi", default=default_choice)
+            return selector, combine, overlay
+        if default_choice is not None:
+            try:
+                indx = content.index(default_choice)
+            except ValueError:
+                indx = 0
+        else:
+            indx = 0
+        selector = st.selectbox("", content, key=f"{key}_sel", index=indx)
+        return [selector], combine, overlay
 
 
 def add_stats_selector(
@@ -94,6 +107,9 @@ def add_stats_selector(
     default_selector="one",
     choices=None,
     key="",
+    default_choice=None,
+    combine=True,
+    overlay=True,
 ):
     if label is None:
         label = file_name.split("_")[0] + " Statistics"
@@ -136,9 +152,10 @@ def add_stats_selector(
             key=f"{file_name}_{key}",
             none=True,
             default=default_selector,
-            check=True,
-            check2=True,
+            check=combine,
+            check2=overlay,
             default_check=xvar == "step",
+            default_choice=default_choice,
         )
     return world_stats, selected_world_stats, combine_world_stats, overlay_world_stats
 
@@ -221,7 +238,7 @@ def line_with_band(fig, stats, xvar, yvar, color, i, color_val=None, ci_level=DE
     if fig is None:
         fig = go.Figure()
     if not isinstance(color_val, str):
-        color_val = str(color_val) 
+        color_val = str(color_val)
     yname = yvar if not color_val else f"{color_val}:{yvar}" if ":" not in color_val else color_val
     fig.add_trace(go.Scatter(x=x, y=y, name=yname, line_color=colors[i % len(colors)]))
     clr = str(tuple(plotly.colors.hex_to_rgb(colors[i % len(colors)]))).replace(" ", "")
@@ -356,7 +373,7 @@ def add_stats_display(
     return cols, end_col
 
 
-def plot_network(nodes, node_weights=None, color_title=None, edges=[], title="", edge_weights=True, edge_colors=True):
+def plot_network(fields, nodes, node_weights=None, color_title=None, edges=[], title="", edge_weights=True, edge_colors=True):
     edge_x = []
     edge_y = []
     annotations = []
@@ -376,7 +393,7 @@ def plot_network(nodes, node_weights=None, color_title=None, edges=[], title="",
         fraction = random.random() * 0.4 + 0.2
         slope = (y1-y0) / (x1 - x0)
         dx = fraction * (x1 - x0)
-        x = x0 +dx 
+        x = x0 +dx
         y = slope * dx + y0
         # x = min(x0, x1) + fraction * (max(x0, x1) - min(x0, x1))
         # y = min(y0, y1) + fraction * (max(y0, y1) - min(y0, y1))
@@ -417,13 +434,20 @@ def plot_network(nodes, node_weights=None, color_title=None, edges=[], title="",
     node_y = []
     node_info = []
     node_w = []
-    for node in nodes:
+    for node in nodes.keys():
         n= nodes[node]
         x, y = n["pos"]
         node_x.append(x)
         node_y.append(y)
-        node_info.append((n.get('cost', 0), n.get('final_score', 0)))
+        node_info.append(tuple(n.values()))
         node_w.append(n.get(node_weights, 1))
+
+    hovertemplate=""
+    if len(nodes):
+        for i, k in enumerate(fields):
+            if k in ("id", "name", "pos", "is_default"):
+                continue
+            hovertemplate += f"<b>{k}</b>:%{{customdata[{i}]}}<br>"
     node_trace = go.Scatter(
         name="",
         x=node_x,
@@ -431,7 +455,7 @@ def plot_network(nodes, node_weights=None, color_title=None, edges=[], title="",
         mode="markers+text",
         customdata=node_info,
         textposition="top center",
-        hovertemplate="cost:%{customdata[0]:.1f}<br>score:%{customdata[1]:.3f} ",
+        hovertemplate=hovertemplate,
         marker=dict(
             showscale=True,
             # colorscale options
@@ -481,7 +505,7 @@ def plot_network(nodes, node_weights=None, color_title=None, edges=[], title="",
     return fig
 
 
-def score_distribution(selected_worlds, selected_agents, selected_types, data):
+def score_distribution(selected_worlds, selected_agents, selected_types, data, parent=st.sidebar):
     # st.write(data["a"])
     # st.write(data["a"].groupby(["world", "type", "input_product"])["final_score"].count())
     expander = st.beta_expander("Score Distribution")
@@ -539,3 +563,54 @@ def score_distribution(selected_worlds, selected_agents, selected_types, data):
     scores = data.groupby("agent")["final_score"].count().reset_index()
     scores = scores.rename(columns=dict(final_score="count"))
     col2.plotly_chart(px.bar(scores, x="agent", y="count"))
+
+def score_factors(selected_worlds, selected_agents, selected_types, data, parent=st.sidebar):
+    expander = st.beta_expander("Final Score Factors")
+    col1, col2, col3 = expander.beta_columns(3)
+    show_counts = col2.checkbox("Show counts only", value=False)
+    is_type = col1.checkbox("Agent Types", value=True, key=f"is_type_check_factors")
+    no_default = col3.checkbox("Ignore Default Agents", value=True, key=f"no_default_agents_factors")
+    selected = selected_types if is_type else selected_agents
+    col = "type" if is_type else "name"
+    data = data["a"].loc[data["a"].world.isin(selected_worlds), :]
+    data["config"] = data["world"].str.split("_").str[0]
+    data["n"] = 1
+    target = "final_score" if not show_counts else "n"
+    if no_default:
+        data = data.loc[~data.is_default, :]
+        data = data.drop("is_default", axis=1)
+    data = data.drop("id", axis=1)
+    if is_type:
+        data = data.drop("name", axis=1)
+    else:
+        data = data.drop("type", axis=1)
+    data = data.rename(columns=dict(name="agent", type="agent"))
+    data = data.loc[data.agent.isin(selected), :]
+    cols = [_ for _ in data.columns if _ not in ("agent", "id", target)]
+    agents = expander.multiselect("Agents", data.agent.unique())
+    if len(agents) > 0:
+        data = data.loc[data["agent"].isin(agents)]
+    facet_col = expander.selectbox("Facet Columns", ["none"] + cols, index=0)
+    if facet_col == "none":
+        facet_col = None
+    else:
+        data = data.sort_values(facet_col)
+    factors = expander.selectbox("Factors", cols)
+    expander.write(f"**Final score vs {factors}**")
+    tbl = data.groupby(([] if not facet_col else [facet_col]) + ["agent"] + [factors])[target].describe().reset_index().set_index("agent" if not facet_col else ["agent", facet_col])
+    tbl.reset_index(inplace=True)
+    graph_type = expander.selectbox("Graph type", ["Scatter", "Bar", "Box", "Line"])
+
+    fig = None
+    if graph_type == "Scatter":
+        fig = px.scatter(data, x=factors, y=target, color="agent", facet_col=facet_col, facet_col_wrap=3)
+    elif graph_type == "Bar":
+        fig = px.bar(data, x=factors, y=target, color="agent", facet_col=facet_col, facet_col_wrap=3)
+    elif graph_type == "Box":
+        fig = px.box(data, x=factors, y=target, color="agent", facet_col=facet_col, facet_col_wrap=3)
+    elif graph_type == "Line":
+        fig = px.line(tbl, x=factors, y="count" if show_counts else "mean", color="agent", facet_col=facet_col, facet_col_wrap=3, facet_row_spacing=0.01)
+
+    if fig:
+        expander.plotly_chart(fig)
+    expander.dataframe(tbl)
